#ifndef ES_H
#define ES_H 

#include<cmath>
#include<algo.h>
#include <vector.h>
#include<iostream.h>
#include "./fitness.h"
#include "./individual.h"

using namespace std;

// ****************************************************************
//  ES (base version with isotropic mutations)
// ****************************************************************

vector<Individual> mu_rho_la_ES( 
  const vector<Individual>& parent_pop_init, // initial parent population 
  size_t mu,                                 // of size mu
  size_t rho,                                // mixing number (family size)
  size_t lambda,                             // number of offspring
  char obj_reco,                      // type of object recombination  "I"|"D"
  char strat_reco,                    // type of strategy param. reco. "I"|"D"
  char sel_type,                      // selection type "+"|","
  double tau,                         // sigma learning parameter 
  size_t max_number_of_generations,   // stop condition: max. no. of generat.
  double stop_cond_sigma )            // stop condition: minimal sigma_0
//
{ size_t parent_pop_size = parent_pop_init.size();
  size_t o_dim = parent_pop_init[0].object_parameter.size();
//
  Individual offspring;
  vector<Individual> offspring_pop(lambda);
  vector<Individual> family(rho);
  vector<Individual> parent_pop = parent_pop_init;
//
  if( sel_type == '+' ) 
    offspring_pop.insert(offspring_pop.end(), mu, offspring);
//
  size_t gen_count = 1;
  do 
  { for(size_t l=0; l < lambda; l++)
    { family = mate(parent_pop, rho);
      offspring = recombine(family, obj_reco, strat_reco); 
      offspring.mutate(tau);
      offspring.evaluate();
      offspring_pop[l] = offspring;
    }
    if( sel_type == '+' ) 
      for( size_t i=0; i < mu; i++) offspring_pop[lambda+i] = parent_pop[i];

    partial_sort_copy( offspring_pop.begin(), offspring_pop.end(), 
                       parent_pop.begin(), parent_pop.end(), 
                       Compare_Individual() );
//
    cout << gen_count << "  " << parent_pop[0].fitness << "  " 
         << parent_pop[0].strategy_parameter << endl;
//
    ++gen_count;
  } while( gen_count <= max_number_of_generations &&
           parent_pop[0].strategy_parameter >= stop_cond_sigma);

  return parent_pop;
}
// ****************************************************************
#endif
