#ifndef INDIVIDUAL_H
#define INDIVIDUAL_H 

#include<cmath>
#include<vector.h>
#include<iostream.h>
#include "./random_generators.h"

using namespace std;

class Individual 
{ 
  public:
    double fitness;
    vector<double> object_parameter;
    double strategy_parameter;
    void evaluate();
    void mutate(double);
};

// ****************************************************************

void Individual::evaluate()
{ double objective_function( const vector<double> &);
  fitness = objective_function(object_parameter);
}

// ****************************************************************

void Individual::mutate(double tau=0.3)
{ size_t o_dim = object_parameter.size();
  strategy_parameter 
   = strategy_parameter*exp(tau*gaussSP.SampleNormalZeroOne());
  for(size_t i=0; i < o_dim; i++)
    object_parameter[i] = object_parameter[i] 
                          + strategy_parameter*gaussOP.SampleNormalZeroOne();
}

// ****************************************************************
// ****************************************************************

vector<Individual> mate(const vector<Individual>& par_pop, size_t rho)
{ size_t mu = par_pop.size();
  if( mu <= rho ) return par_pop; 
  else
  { vector<Individual> temp(rho);
    for(size_t r = 0; r < rho; r++) 
      temp[r] = par_pop[ (size_t) 
                         uni1.SampleDiscreteUnifMinMax(0, (size_t) mu-1) ]; 
    return temp;
  }
}

// ****************************************************************
// ****************************************************************

bool operator < (const Individual& a, const Individual& b)
{ return (a.fitness < b.fitness); }

// ****************************************************************

struct Compare_Individual
{
  bool operator() (const Individual& x, const Individual& y)
  { return (x < y); };
};

// ****************************************************************
// ****************************************************************

Individual recombine( const vector<Individual>& family, 
                      char obj_reco = 'D', char strat_reco = 'I' )
{ size_t rho = family.size();
  size_t o_dim = family[0].object_parameter.size();
  Individual recombinant;
  vector<double> new_object_parameter(o_dim);
  double new_strategy_parameter, sum;
// strategy parameter reco
  switch(strat_reco)
  { case 'I' : case 'i' :
      sum = 0; 
      for( size_t m=0; m < rho; m++ ) sum = sum + family[m].strategy_parameter;
      new_strategy_parameter = sum/((double) rho);
      break;
    case 'D' : case 'd' :
      new_strategy_parameter = family[(size_t) 
        uni2.SampleDiscreteUnifMinMax(0, (size_t) rho-1)].strategy_parameter;
      break;
    default : 
      cout << " Wrong strategy reco type: " 
           << strat_reco << "  -> Stop" << endl;
      exit(-1);
  }
  recombinant.strategy_parameter = new_strategy_parameter;
// object parameter reco
  switch(obj_reco)
  { case 'I' : case 'i' :
      for( size_t i = 0; i < o_dim; i++ )
      { sum = 0; 
        for( size_t m = 0; m < rho; m++ ) 
          sum = sum + family[m].object_parameter[i];
        new_object_parameter[i] = sum/((double) rho);
      }; break;
    case 'D' : case 'd' :
      for( size_t i = 0; i < o_dim; i++ )
        new_object_parameter[i] = family[(size_t) 
        uni2.SampleDiscreteUnifMinMax(0, (size_t) rho-1)].object_parameter[i];
      break;
    default : 
      cout << " Wrong object reco type: " 
           << obj_reco << "  -> Stop" << endl;
      exit(-1);
  }
  recombinant.object_parameter = new_object_parameter;
  return recombinant;
}
 
// ****************************************************************

#endif
